"""Custom API configuration. The base class is simple, containing a BASE_URL and
optional API key & token.

For more customized APIs, requiring more properties, create custom classes that
inherit from APISettings.

"""
from __future__ import annotations

from pathlib import Path
from typing import Optional

from pydantic import BaseModel, Extra, Field, ValidationError, validator
from pydantic_settings import BaseSettings

# THIS_DIR = Path(__file__).parent


class APISettings(BaseSettings):
    BASE_URL: Optional[str] = Field(default=None, env="BASE_URL")
    API_KEY: Optional[str] = Field(default=None, env="API_KEY")
    API_TOKEN: Optional[str] = Field(default=None, env="TOKEN")

    class Config:
        env_file = f"settings/env_files/api.env"
        extra = Extra.allow

    @validator("API_KEY", "API_TOKEN")
    def validate_api_key(cls, v) -> str:
        """If no value set/detected in env, return None."""
        return v or None



from __future__ import annotations

from pathlib import Path

## Import custom classes
from red_config.extra.api_config import APISettings

from pydantic import BaseModel, Field, ValidationError, validator
from pydantic_settings import BaseSettings

allowed_log_levels = ["DEBUG", "CRITICAL", "WARNING", "ERROR", "INFO"]


class AppSettings(BaseSettings):
    APP_TITLE: str = Field(default="Default App Title", env="APP_TITLE")
    APP_DESCRIPTION: str = Field(
        default="Default app description", env="APP_DESCRIPTION"
    )
    APP_VERSION: str = Field(default="0.0.1", env="APP_VERSION")

    class Config:
        env_file = f"settings/env_files/.env"

    @validator("APP_TITLE")
    def validate_app_title(cls, v) -> str:
        """If no value set/detected in env, return None."""
        return v or "Default App Title"

    @validator("APP_DESCRIPTION")
    def validate_app_description(cls, v) -> str:
        """If no value set/detected in env, return None."""
        return v or "Default app description"

    @validator("APP_VERSION")
    def validate_app_ver(cls, v) -> str:
        """If no value set/detected in env, return None."""
        return v or "0.0.1"


class LoggingSettings(BaseSettings):
    LOG_LEVEL: str = Field(default="INFO", env="LOG_LEVEL")

    @validator("LOG_LEVEL")
    def valid_log_level(cls, v) -> str:
        if not v:
            v = "INFO"

        if v not in allowed_log_levels:
            raise ValidationError(
                f"Invalid log level [{v}]. Must be one of {allowed_log_levels}"
            )

        return v

    class Config:
        env_file = f"settings/env_files/logging.env"


# app_settings = AppSettings()
# logging_settings = LoggingSetting()
# api_settings = APISettings()



from __future__ import annotations

from .config import APISettings, AppSettings, LoggingSettings
from .config_utils import config_loader

app_settings = AppSettings()
logging_settings = LoggingSettings()
api_settings = APISettings()
